#!@PERL@

# --8<--8<--8<--8<--
#
# Copyright (C) 2011 Smithsonian Astrophysical Observatory
#
# This file is part of make-world
#
# make-world is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# make-world is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--


package MakeWorld::Package::LuaRock;

use strict;
use warnings;

use Carp;

use File::Basename;
use File::Spec::Functions qw/ catdir catfile /;
use File::Temp qw( tempdir );
use File::Copy;
use File::Path;
use Cwd qw( getcwd );

use MakeWorld::Global;
use MakeWorld::Log;
use MakeWorld::Run;
use MakeWorld::Utils qw[ find_exe ];


use base 'MakeWorld::Package';

sub stages { [ qw[ install ] ] }

sub vtpl {

    return {
	    subdir => { required => 0 },
	    arg => { required => 0 },
	   }

}

sub matches_contents { 0; }

sub matches_archive {

    my ( $class, $archive ) = @_;

    my $ext = qr /\.src\.rock$/;
    return unless $archive =~ $ext;

    return fileparse( $archive, $ext );

}

sub extract_version {

    my ( $class, $version ) = @_;

    my @version = $version =~ /^
			      (\d+)
			      (?:\.(\d+))?
			      (?:\.(\d+))?
			      (\w+)?
			      -
			      (\d+)
			      $/x;

    die( "unable to extract version information from $version\n" )
      unless @version;

    return join( '.', map { $_ // 0 } @version[0,1,2] ),
      $version[3], $version[4];
}

sub extract
{
    my ( $self )  = @_;

    my $base = $self->archive->{base};
    my $file = $self->archive->{file};

    my $cwd = getcwd;

    $self->{luarocks} = find_exe( 'luarocks' );
    # directory where the archive is extracted
    my $extdir;


    # this is the name of the directory that'll get renamed as $base.
    # either $extdir or $extdir/subdir
    my $srcdir;

    if ( $self->ok_to_run( 'extract' ) )
    {
	# FIXME: this is a duplicate of MakeWorld::Archive::extract. merge 
	# this somehow
	eval {

	    # extract archive into its own directory.  this allows
	    # processing of archives which don't unpack into
	    # a properly named and versioned directory

	    unless ( $opt{dryrun} )
	    {
		$extdir = tempdir( DIR => $cwd );
		chdir( $extdir )
		  or die( "unable to chdir to $extdir\n" );
	    }

	    msg( MakeWorld::Log::Notice, "extracting $file" );


	    eval {
		my $rd = $self->logredir( 'extract' );

		# luarocks tries to optimize determination of the current
		# working directory by using the PWD environment variable,
		# but that's not correct.  remove the temptation...
		local %ENV = %ENV;
		delete $ENV{PWD};
		run( $self->{luarocks}, 'unpack', $file )
		  or die( "error extracting package to $extdir\n" );
	    };
	    if ( my $err = $@ ) {
		my $logfile = $self->logfile( 'extract' );
		die ( $err, defined $logfile ? "for more info see\n$logfile\n" : () );
	    }

	    if ( ! $opt{dryrun} )
	    {
		opendir( my $dh, '.')
		  or die( 'unable to open current directory?');
		# now, what's in the current directory?
		my @dircontents = grep { ! /^\./ } readdir( $dh );
		closedir( $dh );
		# only one thing and its a directory. cool.
		if ( 1 == @dircontents && -d $dircontents[0] )
		{
		    $srcdir = catdir( $extdir, $dircontents[0] );
		}

		# more than one entry. rename $extdir to $base
		else
		{
		    die( "extracted more than expected: got ", join(", ", @dircontents ), "\n" );
		}
	    }

	    chdir( $cwd );
	    # remove build directory if it exists
	    if ( -d $base )
	    {
		msg( MakeWorld::Log::Info, "removing existing build in $cwd/$base" );
		rmtree( $base )
		  unless $opt{dryrun};
	    }

	    unless ( $opt{dryrun} )
	    {
		move( $srcdir, $base )
		  or die( "error renaming $srcdir to $base\n" );
	    }

	};

	my $error = $@;
	chdir( $cwd );

	# remove extdir if it hasn't been renamed to $base
	rmtree( $extdir ) if defined $extdir && -d $extdir;

	die $@ if $@;
    }

    # now that it's extracted, find the rockspec and the actual directory
    # containing the code
    return $self->_find_srcdir( $cwd, $base );

}

sub _find_srcdir {

    my ( $self, $cwd, $base ) = @_;

    my $srcdir = catdir( $cwd, $base );

    chdir( $srcdir )
      or die( "unable to change directory to $srcdir\n" );

    my ( @rockspec ) = glob( '*.rockspec' );

    die( "expected one rockspec file, got ", scalar @rockspec, "\n" )
      unless 1 == @rockspec;

    $self->{rockspec} = catfile( $srcdir, $rockspec[0] );

    my $subdir = $self->option('subdir');

    if ( ! defined $subdir ) {
	opendir( my $dh, '.')
	    or die( 'unable to open current directory?');
	my @dirs = grep { ! /^\./ && -d $_ } readdir( $dh );
	closedir( $dh );

	die( "can't figure out which directory contains the code\n" )
	    if @dirs > 1;

	$subdir = $dirs[0];
    }

    return catdir( $cwd, $base, $subdir );
}


sub install
{
    my $self = shift;

    my $options = $self->option('install');

    $self->SUPER::build && return;

    # luarocks tries to optimize determination of the current
    # working directory by using the PWD environment variable,
    # but that's not correct.  remove the temptation...
    local %ENV = %ENV;
    delete $ENV{PWD};

    run( $self->{luarocks}, $self->option('arg'), 'make', $self->{rockspec} )
	or die( "error building and installing package\n" );

    return;
}


1;
